/** @file
    Decoder for Bresser Weather Center 7-in-1.

    Copyright (C) 2019 Christian W. Zuckschwerdt <zany@triq.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

#include "decoder.h"

/**
Decoder for Bresser Lightning, outdoor sensor.

https://github.com/merbanan/rtl_433/issues/2140

Preamble:

  aa 2d d4

Observed length depends on reset_limit.
The data has a whitening of 0xaa.


First two bytes are an LFSR-16 digest, generator 0x8810 key 0xabf9 with a final xor 0x899e
*/

static int bresser_lightning_decode(r_device *decoder, bitbuffer_t *bitbuffer)
{
    uint8_t const preamble_pattern[] = {0xaa, 0x2d, 0xd4};
    bitbuffer_print(bitbuffer);

    data_t *data;
    uint8_t msg[24];
 

    if (bitbuffer->num_rows != 1 || bitbuffer->bits_per_row[0] < 25) {
        decoder_logf(decoder, 2, __func__, "to few bits (%u)", bitbuffer->bits_per_row[0]);
        return DECODE_ABORT_LENGTH; // unrecognized
    }

    unsigned start_pos = bitbuffer_search(bitbuffer, 0, 0,
            preamble_pattern, sizeof(preamble_pattern) * 8);
    start_pos += sizeof(preamble_pattern) * 8;

    if (start_pos >= bitbuffer->bits_per_row[0]) {
        decoder_log(decoder, 2, __func__, "preamble not found");
         return DECODE_ABORT_EARLY; // no preamble found
    }
   //if (start_pos + sizeof (msg) * 8 >= bitbuffer->bits_per_row[0]) {
  if (start_pos + 1*8 >= bitbuffer->bits_per_row[0]) {
    decoder_logf(decoder, 2, __func__, "message too short (%u)", bitbuffer->bits_per_row[0] - start_pos);
       return DECODE_ABORT_LENGTH; // message too short
 }
    bitbuffer_extract_bytes(bitbuffer, 0, start_pos, msg, sizeof (msg) * 8);
    decoder_log_bitrow(decoder, 2, __func__, msg, sizeof(msg) * 8, "MSG");


  //data whitening
for (unsigned i = 0; i < sizeof (msg); ++i) {
   msg[i] ^= 0xaa;
   }
    decoder_log_bitrow(decoder, 2, __func__, msg, sizeof(msg) * 8, "XOR");

    // LFSR-16 digest, generator 0x8810 key 0xabf9 with a final xor 0x899e
    int chk    = (msg[0] << 8) | msg[1];
      int digest = lfsr_digest16(&msg[2], 8, 0x8810, 0xabf9);
         fprintf(stderr, "DIGEST %04x vs %04x (%04x) \n", chk, digest, chk ^ digest);
      if (((chk ^ digest) != 0x899e) && ((chk ^ digest) != 0x8b9e)) {
        decoder_logf(decoder, 2, __func__, "Digest check failed %04x vs %04x (%04x)", chk, digest, chk ^ digest);
       
         
     return DECODE_FAIL_MIC;
    }
     int id       = (msg[2] << 8) | (msg[3]);
    
    for (unsigned i = 0; i < sizeof (msg); ++i) {
   fprintf(stderr, "msg %u %x   |  ",i,msg[i]); //for debugging ?
 
   }
       
    int ctr     = msg[4] | (msg[5] >>4); // is this right? should be a 12H but how to do it?
    int battery_low = (msg[5] & 0x0f); // 8 Full 0 Empty
    int frage1 = (msg[6]); //what ever this value is
    int kilometer = (msg[7] ) ;
    int frage2= msg[8]  |msg[9]; //what ever this value is
                
    
    /* clang-format off */
    data = data_make(
            "model",            "",             DATA_STRING, "Bresser_lightning Test",
            "id",               "",             DATA_INT,    id,
            "Frage1",    	 "?",           DATA_INT, frage1,
            "Kilometer",     "Kilometer",   DATA_INT, kilometer,
            "CTR",  		   "CTR",    	DATA_INT,    ctr,
            "Frage2",          "??",         DATA_INT, frage2,
            "mic",              "Integrity",    DATA_STRING, "CRC",
            "battery_low",		"Battery Low"	,DATA_INT, !battery_low,
            NULL);
    /* clang-format on */

    decoder_output_data(decoder, data);
     
    return 1;
}

static char *output_fields[] = {
        "model",
        "id",
        "battery_low",
        "Frage1"
        "Kilometer"
        "Frage2"
        "mic",
        NULL,
};

r_device bresser_lightning = {
        .name        = "Bresser Lightning",
        .modulation  = FSK_PULSE_PCM,
        .short_width = 120,
        .long_width  = 120,
        .reset_limit = 20000,
        .decode_fn   = &bresser_lightning_decode,
        .fields      = output_fields,
};
